<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if(!function_exists('curl_init'))
{
    throw new Exception('CURL para PHP no está instalado o activado. PayCash requiere de este componente para funcionar.');
}

if(!defined('_PS_VERSION_'))
{
	exit;
}

class PayCashPrestashop extends PaymentModule
{	
	public function __construct()
    {
		$this->name = 'PayCashPrestashop';
		$this->tab = 'payments_gateways';
		$this->version = '1.0.0';
		$this->author = 'PayCash';
        $this->controllers = array('payment', 'validation');
		$this->currencies = true;
		$this->currencies_mode = 'checkbox';
		$this->bootstrap = true;
		$this->displayName = 'Pago en efectivo con PayCash';
        $this->description = 'PayCash la red más grande y confiable de Latam para procesar pagos en efectivo y transferencias bancarias que permite atraer nuevos clientes y aumentar tus ingresos mediante una experiencia de pago flexible que se adapta a tus necesidades de cobro.';
		$this->confirmUninstall = 'Deseas desinstalar este módulo?';
        $this->ps_versions_compliancy = array('min' => '1.7.0', 'max' => _PS_VERSION_);
		
		parent::__construct();
	}
	
	public function install()
    {
		if(!parent::install() || !$this->crearActualizarEstadoOrden()
			|| !$this->registerHook('paymentOptions')
			|| !$this->registerHook('paymentReturn')
			|| !$this->registerHook('updateOrderStatus')
			&& Configuration::updateValue('paycashps_enabled', false)
			&& Configuration::updateValue('paycashps_testmode', true)
			&& Configuration::updateValue('paycashps_title', 'Paga en efectivo con PayCash')
			&& Configuration::updateValue('paycashps_test_key', '5d9d90c5013111ecaf8b0afe8920d1ea')
			&& Configuration::updateValue('paycashps_production_key', '5d9d90c5013111ecaf8b0afe8920d1ea')
			&& Configuration::updateValue('paycashps_country', 'MEX')
			&& Configuration::updateValue('paycashps_validity', '3')
			&& Configuration::updateValue('paycashps_description', 'Prueba de texto')
			&& Configuration::updateValue('paycashps_instructions', '<br /><br /><b>Realiza tu pago en efectivo en cualquiera de los establecimientos afiliados a PayCash con la referencia mostrada a continuación: </b></br><br /><img src=https://lh3.googleusercontent.com/drive-viewer/AITFw-z5Q3PC7PLBoWfL7QSOT7CTRstMPabsTq3EGHc8qnGihv6QzfeXwo1O97jZUukG_L1_YKZY4lu4xbtQsyp9ns_aKTbBoQ=s1600>
<div style="text-align: center"><a href=https://paycashglobal.com/buscador.php target="_blank"><img src=https://lh3.googleusercontent.com/drive-viewer/AITFw-zyIrk2x7uaQ4biCFfGwRw_ytVa_CfS7oGvhpQ8DLilyrCFFwC899Iguw-is_HRH3rMSQnC0EDOO2-hd2OMWNBkVAh04Q=s1600 style="text-align: center;max-width:40%;!important"></a></div>')
			|| !$this->crearTablaOrdenReferencia()
		)
		{
			return false;
		}
		return true;
	}

	private function crearActualizarEstadoOrden()
	{
		$estadoOrdenPayCash = 'PS_OS_PAYCASH_EN_ESPERA_PAGO';
		$nombreEstadoOrdenPayCash = 'En espera de pago por PayCash';
		
		if (!Configuration::get($estadoOrdenPayCash) || Configuration::get($estadoOrdenPayCash) == 'ERROR')
		{
			$order_state = new OrderState();
			$order_state->send_email = false;
			$order_state->module_name = $this->name;
			$order_state->template = array((int)Configuration::get('PS_LANG_DEFAULT') => pSQL($this->l("paycash_en_espera_de_pago")));//array_fill(0, 10, "paycash");
			$order_state->invoice = false;
			$order_state->color = '#34209E';
			$order_state->lovable = true;
			$order_state->shipped = false;
			$order_state->unremovable = true;
			$order_state->delivery = false;
			$order_state->hidden = false;
			$order_state->paid = false;
			$order_state->deleted = false;
			$order_state->name = array((int)Configuration::get('PS_LANG_DEFAULT') => pSQL($this->l($nombreEstadoOrdenPayCash)));
			
			if($order_state->add())
			{
				Configuration::updateValue($estadoOrdenPayCash, $order_state->id);
				
				if(class_exists('Logger'))
					Logger::addLog("Estado de orden ".$nombreEstadoOrdenPayCash." agregado con ID ".$order_state->id, 1, null, 'OrderState', $order_state->id, true);
				
				$directorioPlantillas = _PS_MAIL_DIR_;
				
				$nombreArchivoPlantillaHTML = 'paycash_en_espera_de_pago.html';
				$nombreArchivoPlantillaTXT = 'paycash_en_espera_de_pago.txt';
				$nombreArchivoPlantillaCancelacionHTML = 'paycash_cancelacion.html';
				$nombreArchivoPlantillaCancelacionTXT = 'paycash_cancelacion.txt';
				$nombreArchivoPlantillaPagoHTML = 'paycash_pago.html';
				$nombreArchivoPlantillaPagoTXT = 'paycash_pago.txt';
				$plantillaHTML = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaHTML;
				$plantillaTXT = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaTXT;
				$plantillaCancelacionHTML = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaCancelacionHTML;
				$plantillaCancelacionTXT = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaCancelacionTXT;
				$plantillaPagoHTML = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaPagoHTML;
				$plantillaPagoTXT = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaPagoTXT;
				
				if(file_exists($directorioPlantillas.'mx'))
				{
					$plantillaHTMLARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaHTML;
					$plantillaTXTARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaTXT;
					$plantillaCancelacionHTMLARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaCancelacionHTML;
					$plantillaCancelacionTXTARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaCancelacionTXT;
					$plantillaPagoHTMLARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaPagoHTML;
					$plantillaPagoTXTARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaPagoTXT;
					
					try
					{
						Tools::copy($plantillaHTML, $plantillaHTMLARemplazar);
						Tools::copy($plantillaTXT, $plantillaTXTARemplazar);
						Tools::copy($plantillaCancelacionHTML, $plantillaCancelacionHTMLARemplazar);
						Tools::copy($plantillaCancelacionTXT, $plantillaCancelacionTXTARemplazar);
						Tools::copy($plantillaPagoHTML, $plantillaPagoHTMLARemplazar);
						Tools::copy($plantillaPagoTXT, $plantillaPagoTXTARemplazar);
					}
					catch(Exception $e)
					{
						$error = $e->getMessage().'. ';
						
						if(class_exists('Logger'))
							Logger::addLog('Error al agregar plantillas MX. '.$error, 1, null, null, null, true);
					}
				}
				
				if(file_exists($directorioPlantillas.'es'))
				{
					$plantillaHTMLARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaHTML;
					$plantillaTXTARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaTXT;
					$plantillaCancelacionHTMLARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaCancelacionHTML;
					$plantillaCancelacionTXTARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaCancelacionTXT;
					$plantillaPagoHTMLARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaPagoHTML;
					$plantillaPagoTXTARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaPagoTXT;
					
					try
					{
						Tools::copy($plantillaHTML, $plantillaHTMLARemplazar);
						Tools::copy($plantillaTXT, $plantillaTXTARemplazar);
						Tools::copy($plantillaCancelacionHTML, $plantillaCancelacionHTMLARemplazar);
						Tools::copy($plantillaCancelacionTXT, $plantillaCancelacionTXTARemplazar);
						Tools::copy($plantillaPagoHTML, $plantillaPagoHTMLARemplazar);
						Tools::copy($plantillaPagoTXT, $plantillaPagoTXTARemplazar);
					}
					catch(Exception $e)
					{
						$error = $e->getMessage().'. ';
						
						if(class_exists('Logger'))
							Logger::addLog('Error al agregar plantillas ES. '.$error, 1, null, null, null, true);
					}
				}
				
				if(file_exists($directorioPlantillas.'en'))
				{
					$plantillaHTMLARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaHTML;
					$plantillaTXTARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaTXT;
					$plantillaCancelacionHTMLARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaCancelacionHTML;
					$plantillaCancelacionTXTARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaCancelacionTXT;
					$plantillaPagoHTMLARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaPagoHTML;
					$plantillaPagoTXTARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaPagoTXT;
					
					try
					{
						Tools::copy($plantillaHTML, $plantillaHTMLARemplazar);
						Tools::copy($plantillaTXT, $plantillaTXTARemplazar);
						Tools::copy($plantillaCancelacionHTML, $plantillaCancelacionHTMLARemplazar);
						Tools::copy($plantillaCancelacionTXT, $plantillaCancelacionTXTARemplazar);
						Tools::copy($plantillaPagoHTML, $plantillaPagoHTMLARemplazar);
						Tools::copy($plantillaPagoTXT, $plantillaPagoTXTARemplazar);
					}
					catch(Exception $e)
					{
						$error = $e->getMessage().'. ';
						
						if(class_exists('Logger'))
							Logger::addLog('Error al agregar plantillas EN. '.$error, 1, null, null, null, true);
					}
				}
				
				if(class_exists('Logger'))
					Logger::addLog("Plantillas PayCash de correo electronico agregadas", 1, null, null, null, true);
			}
			else
			{
				Configuration::updateValue($estadoOrdenPayCash, 'ERROR');
				
				if(class_exists('Logger'))
					Logger::addLog('Error al crear el estado de orden '.$nombreEstadoOrdenPayCash, 1, null, null, null, true);	
				
				return false;
			}
		}
		else
		{
			$estateID = Configuration::get($estadoOrdenPayCash);
			$order_state = new OrderState($estateID);
			$order_state->name = array((int)Configuration::get('PS_LANG_DEFAULT') => pSQL($this->l($nombreEstadoOrdenPayCash)));
			$order_state->color = '#34209E';
			$order_state->send_email = false;
			$order_state->unremovable = true;
			$order_state->save();
			
			if(class_exists('Logger'))
				Logger::addLog('Estado de orden '.$nombreEstadoOrdenPayCash.' con ID '.$order_state->id." actualizado", 1, null, 'OrderState', $estateID, true);
			
			$directorioPlantillas = _PS_MAIL_DIR_;
				
			$nombreArchivoPlantillaHTML = 'paycash_en_espera_de_pago.html';
			$nombreArchivoPlantillaTXT = 'paycash_en_espera_de_pago.txt';
			$nombreArchivoPlantillaCancelacionHTML = 'paycash_cancelacion.html';
			$nombreArchivoPlantillaCancelacionTXT = 'paycash_cancelacion.txt';
			$nombreArchivoPlantillaPagoHTML = 'paycash_pago.html';
			$nombreArchivoPlantillaPagoTXT = 'paycash_pago.txt';
			$plantillaHTML = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaHTML;
			$plantillaTXT = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaTXT;
			$plantillaCancelacionHTML = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaCancelacionHTML;
			$plantillaCancelacionTXT = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaCancelacionTXT;
			$plantillaPagoHTML = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaPagoHTML;
			$plantillaPagoTXT = _PS_MODULE_DIR_.$this->name.'/templates/'.$nombreArchivoPlantillaPagoTXT;
			
			if(file_exists($directorioPlantillas.'mx'))
			{
				$plantillaHTMLARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaHTML;
				$plantillaTXTARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaTXT;
				$plantillaCancelacionHTMLARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaCancelacionHTML;
				$plantillaCancelacionTXTARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaCancelacionTXT;
				$plantillaPagoHTMLARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaPagoHTML;
				$plantillaPagoTXTARemplazar = $directorioPlantillas.'mx/'.$nombreArchivoPlantillaPagoTXT;
					
				try
				{
					Tools::copy($plantillaHTML, $plantillaHTMLARemplazar);
					Tools::copy($plantillaTXT, $plantillaTXTARemplazar);
					Tools::copy($plantillaCancelacionHTML, $plantillaCancelacionHTMLARemplazar);
					Tools::copy($plantillaCancelacionTXT, $plantillaCancelacionTXTARemplazar);
					Tools::copy($plantillaPagoHTML, $plantillaPagoHTMLARemplazar);
					Tools::copy($plantillaPagoTXT, $plantillaPagoTXTARemplazar);
				}
				catch(Exception $e)
				{
					$error = $e->getMessage().'. ';
					
					if(class_exists('Logger'))
						Logger::addLog('Error al agregar plantillas MX. '.$error, 1, null, null, null, true);
				}
			}
			
			if(file_exists($directorioPlantillas.'es'))
			{
				$plantillaHTMLARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaHTML;
				$plantillaTXTARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaTXT;
				$plantillaCancelacionHTMLARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaCancelacionHTML;
				$plantillaCancelacionTXTARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaCancelacionTXT;
				$plantillaPagoHTMLARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaPagoHTML;
				$plantillaPagoTXTARemplazar = $directorioPlantillas.'es/'.$nombreArchivoPlantillaPagoTXT;
					
				try
				{
					Tools::copy($plantillaHTML, $plantillaHTMLARemplazar);
					Tools::copy($plantillaTXT, $plantillaTXTARemplazar);
					Tools::copy($plantillaCancelacionHTML, $plantillaCancelacionHTMLARemplazar);
					Tools::copy($plantillaCancelacionTXT, $plantillaCancelacionTXTARemplazar);
					Tools::copy($plantillaPagoHTML, $plantillaPagoHTMLARemplazar);
					Tools::copy($plantillaPagoTXT, $plantillaPagoTXTARemplazar);
				}
				catch(Exception $e)
				{
					$error = $e->getMessage().'. ';
					
					if(class_exists('Logger'))
						Logger::addLog('Error al agregar plantillas ES. '.$error, 1, null, null, null, true);
				}
			}
			
			if(file_exists($directorioPlantillas.'en'))
			{
				$plantillaHTMLARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaHTML;
				$plantillaTXTARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaTXT;
				$plantillaCancelacionHTMLARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaCancelacionHTML;
				$plantillaCancelacionTXTARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaCancelacionTXT;
				$plantillaPagoHTMLARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaPagoHTML;
				$plantillaPagoTXTARemplazar = $directorioPlantillas.'en/'.$nombreArchivoPlantillaPagoTXT;
					
				try
				{
					Tools::copy($plantillaHTML, $plantillaHTMLARemplazar);
					Tools::copy($plantillaTXT, $plantillaTXTARemplazar);
					Tools::copy($plantillaCancelacionHTML, $plantillaCancelacionHTMLARemplazar);
					Tools::copy($plantillaCancelacionTXT, $plantillaCancelacionTXTARemplazar);
					Tools::copy($plantillaPagoHTML, $plantillaPagoHTMLARemplazar);
					Tools::copy($plantillaPagoTXT, $plantillaPagoTXTARemplazar);
				}
				catch(Exception $e)
				{
					$error = $e->getMessage().'. ';
					
					if(class_exists('Logger'))
						Logger::addLog('Error al agregar plantillas EN. '.$error, 1, null, null, null, true);
				}
			}
				
			if(class_exists('Logger'))
				Logger::addLog("Plantillas PayCash de correo electronico actualizadas", 1, null, null, null, true);
		}
		
		return true;
	}
	
	public function uninstall()
    {
		if(!parent::uninstall()
			|| !$this->unregisterHook('paymentOptions')
			|| !$this->unregisterHook('paymentReturn')
			|| !$this->unregisterHook('updateOrderStatus')
			|| !Configuration::deleteByName('paycashps_enabled')
			|| !Configuration::deleteByName('paycashps_testmode')
			|| !Configuration::deleteByName('paycashps_title')
			|| !Configuration::deleteByName('paycashps_test_key')
			|| !Configuration::deleteByName('paycashps_production_key')
			|| !Configuration::deleteByName('paycashps_country')
			|| !Configuration::deleteByName('paycashps_validity')
			|| !Configuration::deleteByName('paycashps_description')
			|| !Configuration::deleteByName('paycashps_instructions')
		)
		{
			return false;
		}
		
		return true;
    }
	
	public function getContent()
    {
		$output = '';

		if (Tools::isSubmit('submit'.$this->name))
		{
			$value_paycashps_enabled = Tools::getValue('paycashps_enabled_ActivarDesactivar');
			$value_paycashps_testmode = Tools::getValue('paycashps_testmode_ModoPruebas');
			$value_paycashps_title = (string) Tools::getValue('paycashps_title');
			$value_paycashps_test_key = (string) Tools::getValue('paycashps_test_key');
			$value_paycashps_production_key = (string) Tools::getValue('paycashps_production_key');
			$value_paycashps_country = (string) Tools::getValue('paycashps_country');
			$value_paycashps_validity = (string) Tools::getValue('paycashps_validity');
			$value_paycashps_description = (string) Tools::getValue('paycashps_description');
			$value_paycashps_instructions = (string) Tools::getValue('paycashps_instructions');
			
			if (empty($value_paycashps_title))
				$output = $this->displayError($this->l('El campo Mensaje es obligatorio'));
			
			else if (empty($value_paycashps_test_key))
				$output = $this->displayError($this->l('El campo Test Key es obligatorio'));
			
			else if (empty($value_paycashps_production_key))
				$output = $this->displayError($this->l('El campo Production Key es obligatorio'));
			
			else if (empty($value_paycashps_country))
				$output = $this->displayError($this->l('El campo País es obligatorio'));
			
			/*else if (empty($value_paycashps_validity))
				$output = $this->displayError($this->l('El campo Vigencia es obligatorio'));
			
			else if (!Validate::isInt($value_paycashps_validity))
				$output = $this->displayError($this->l('El campo Vigencia debe contener un valor numérico'));*/
	
			/*else if (empty($value_paycashps_description))
				$output = $this->displayError($this->l('El campo Descripción es obligatorio'));
			
			else if (empty($value_paycashps_instructions))
				$output = $this->displayError($this->l('El campo Instrucciones es obligatorio'));
			*/
			else
			{
				Configuration::updateValue('paycashps_enabled', $value_paycashps_enabled);
				Configuration::updateValue('paycashps_testmode', $value_paycashps_testmode);
				Configuration::updateValue('paycashps_title', $value_paycashps_title);
				Configuration::updateValue('paycashps_test_key', $value_paycashps_test_key);
				Configuration::updateValue('paycashps_production_key', $value_paycashps_production_key);
				Configuration::updateValue('paycashps_country', $value_paycashps_country);
				Configuration::updateValue('paycashps_validity', $value_paycashps_validity);
				Configuration::updateValue('paycashps_description', $value_paycashps_description);
				Configuration::updateValue('paycashps_instructions', $value_paycashps_instructions);
				
				$this->crearActualizarEstadoOrden();
				
				$output = $this->displayConfirmation($this->l('Configuración guardada con éxito.'));
			}
		}

		return $output.$this->displayForm();
    }
	
	public function displayForm()
	{		
		$form = [
			'form' => [
				'legend' => [
					'title' => $this->l('Configuración'),
				],
				'input' => [
					[
						'type' => 'checkbox',
						'label' => $this->l('Activar/Desactivar'),
						'desc' => 'Activa o desactiva el método de pago.',
						'name' => 'paycashps_enabled',
						'required' => true,
						'values' => array(
							'query' => $catalogoActivarDesactivar = array(
								array(
									'id' => 'ActivarDesactivar',
									'name' => 'Activar/Desactivar',
									'val' => '1'
								),
							),
							'id' => 'id',
							'name' => 'name',
							'value' => '1'
						)
					],
					[
						'type' => 'checkbox',
						'label' => $this->l('Modo Pruebas'),
						'desc' => 'Activa el modo pruebas al estar seleccionado.',
						'name' => 'paycashps_testmode',
						'required' => true,
						'values' => array(
							'query' => $catalogoModoPruebas = array(
								array(
									'check_id' => 'ModoPruebas',
									'name' => 'Modo Pruebas',
									'val' => '1'
								)
							),
							'id' => 'check_id',
							'name' => 'name',
							'value' => '1'
						)
					],
					[
						'type' => 'text',
						'label' => $this->l('Mensaje'),
						'name' => 'paycashps_title',
						'desc' => 'Título mostrado en la página de pago.',
						'required' => true,
						'default' => 'Paga en efectivo con PayCash'
					],
					[
						'type' => 'text',
						'label' => $this->l('PayCash API Test Key'),
						'name' => 'paycashps_test_key',
						'desc' => 'Llave API de ambiente de pruebas (sandbox).',
						'required' => true,
					],
					[
						'type' => 'text',
						'label' => $this->l('PayCash API Production Key'),
						'name' => 'paycashps_production_key',
						'desc' => 'Llave API de producción.',
						'required' => true,
					],
					[
						'type' => 'select',
						'label' => $this->l('País'),
						'name' => 'paycashps_country',
						'desc' => 'Seleccione el país.',
						'required' => true,
						'options' => array(
							'query' => $catalogoPaises = array(
								array('codigo' => 'COL', 'name' => 'Colombia'),
								array('codigo' => 'CRI', 'name' => 'Costa Rica'),
								array('codigo' => 'ECU', 'name' => 'Ecuador'),
								array('codigo' => 'MEX', 'name' => 'México'),
								array('codigo' => 'PER', 'name' => 'Perú')
							),
							'id' => 'codigo',
							'name' => 'name',
						)
					],
					[
						'type' => 'text',
						'label' => $this->l('Vigencia (en días)'),
						'name' => 'paycashps_validity',
						'desc' => 'Dejar vacío para no especificar vigencia.',
						'required' => false,
						'default' => '3'
					],/*
					[
						'type' => 'text',
						'label' => $this->l('Descripción'),
						'name' => 'paycashps_description',
						'desc' => 'Descripción mostrada en la página de pago.',
						'required' => true,
						'default' => 'Realiza tu pago en efectivo en cualquiera de los establecimientos afiliados a PayCash con la referencia mostrada a continuación'
					],
					[
						'type' => 'text',
						'label' => $this->l('Instrucciones'),
						'name' => 'paycashps_instructions',
						'desc' => 'Descripción mostrada en correos electrónicos y notificaciones.',
						'required' => true,
						'default' => 'Realiza tu pago en efectivo en cualquiera de los establecimientos afiliados a PayCash con la referencia mostrada a continuación'
					]*/
				],
				'submit' => [
					'title' => $this->l('Guardar los cambios'),
					'class' => 'btn btn-default pull-right',
				],
			],
		];

		$helper = new HelperForm();

		$helper->table = $this->table;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $this->name]);
		$helper->submit_action = 'submit' . $this->name;

		$helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');

		$helper->fields_value['paycashps_enabled_ActivarDesactivar'] = Tools::getValue('paycashps_enabled_ActivarDesactivar', Configuration::get('paycashps_enabled'));
		$helper->fields_value['paycashps_testmode_ModoPruebas'] = Tools::getValue('paycashps_testmode_ModoPruebas', Configuration::get('paycashps_testmode'));
		$helper->fields_value['paycashps_title'] = Tools::getValue('paycashps_title', Configuration::get('paycashps_title'));
		$helper->fields_value['paycashps_test_key'] = Tools::getValue('paycashps_test_key', Configuration::get('paycashps_test_key'));
		$helper->fields_value['paycashps_production_key'] = Tools::getValue('paycashps_production_key', Configuration::get('paycashps_production_key'));
		$helper->fields_value['paycashps_country'] = Tools::getValue('paycashps_country', Configuration::get('paycashps_country'));
		$helper->fields_value['paycashps_validity'] = Tools::getValue('paycashps_validity', Configuration::get('paycashps_validity'));
		$helper->fields_value['paycashps_description'] = Tools::getValue('paycashps_description', Configuration::get('paycashps_description'));
		$helper->fields_value['paycashps_instructions'] = Tools::getValue('paycashps_instructions', Configuration::get('paycashps_instructions'));

		return $helper->generateForm([$form]);
	}
	
	public function hookPaymentOptions($parametros)
    {
        if (!$this->active)
		{
            return;
        }
		
		/*if(!$this->checkCurrency($parametros['cart']))
		{
			return;
		}*/
		
		$enabled = Configuration::get('paycashps_enabled');
		
		if($enabled != '1')
		{
			return;
		}
		
		$description = Configuration::get('paycashps_description');
		
        $formAction = $this->context->link->getModuleLink($this->name, 'validation', array(), true);
		
		$country = Configuration::get('paycashps_country');
		$logo = '';
		
		if($country == 'COL')
			$logo = 'https://paycash-storage.s3.amazonaws.com/PCDocs/COL/colombia.jpg';
		else if($country == 'CRI')
			$logo = 'https://paycash-storage.s3.amazonaws.com/PCDocs/CRI/costarica.jpg';
		else if($country == 'ECU')
			$logo = 'https://paycash-storage.s3.amazonaws.com/PCDocs/ECU/ecuador.jpg';
		else if($country == 'MEX')
			$logo = 'https://paycash-storage.s3.amazonaws.com/PCDocs/MEX/Prestashop_636x86.jpg';
		else if($country == 'PER')
			$logo = 'https://paycash-storage.s3.amazonaws.com/PCDocs/PER/peru.jpg';

        $this->smarty->assign(['action' => $formAction, 'logo' => $logo, 'description' => '']);

        $paymentForm = $this->fetch('module:PayCashPrestashop/views/templates/hook/payment_options.tpl');

        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->displayName)
            ->setAction($formAction)
            ->setForm($paymentForm);

        $payment_options = array(
            $newOption
        );

        return $payment_options;
    }
	
	public function hookPaymentReturn($parametros)
    {
		$test_urlObtenerToken = 'https://api-pcdocs-emisor.paycashglobal.com/v1/authre';
		$produccion_urlObtenerToken = 'https://api-global-emisor.paycashglobal.com/v1/authre?country=';
		$test_urlObtenerReferencia = 'https://api-pcdocs-emisor.paycashglobal.com/v1/reference';
		$produccion_urlObtenerReferencia = 'https://api-global-emisor.paycashglobal.com/v1/reference';
		
        if (!$this->active)
		{
            return;
        }
 
		$testmode = Configuration::get('paycashps_testmode');
		$country = Configuration::get('paycashps_country');
		$vigenciaEnDias = Configuration::get('paycashps_validity');
		$instrucciones = Configuration::get('paycashps_instructions');
		$apiKey = ($testmode) ? Configuration::get('paycashps_test_key') : Configuration::get('paycashps_production_key');
 
		if($apiKey != '')
		{
			if ($parametros['order'] && Validate::isLoadedObject($parametros['order']))
			{
				$ordenID = (int)$parametros['order']->id;
				$orden = new Order($ordenID);
				$totalOrden = $orden->total_paid;
				$idCustomer = $orden->id_customer;
				$ordenReference = $orden->reference;
				$ordenDateAdd = $orden->date_add;
				
				$customer = new Customer($idCustomer);
				$email = $customer->email;
				$customerFirstName = $customer->firstname;
				$customerLastName = $customer->lastname;
				
				$urlObtenerToken = $test_urlObtenerToken;
				$urlObtenerReferencia = $test_urlObtenerReferencia;
				
				if($testmode != '1')
				{
					$urlObtenerToken = $produccion_urlObtenerToken.$country;
					$urlObtenerReferencia = $produccion_urlObtenerReferencia;
				}
				
				$headers = array
				(
					'Content-Type : application/json',
					'key : '.$apiKey
				);
				
				if (!function_exists('curl_version'))
				{
					$this->smarty->assign(['success' => false, 'error' => 'No se puede proceder a conectar con el servicio de PayCash porque no ha habilitado CURL para PHP.']);
				}
				else
				{
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, $urlObtenerToken);
					curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($ch, CURLOPT_HEADER, 0); 
					$data = curl_exec($ch); 
					curl_close($ch); 
					
					$body = json_decode($data);
					
					if($body->ErrorCode != 0)
					{
						$this->smarty->assign(['success' => false, 'error' => $body->ErrorMessage]);
					}
					else
					{
						$token = $body->Authorization;
						
						$parametroFecha = '';
						if(!isset($vigenciaEnDias) || is_null($vigenciaEnDias) || $vigenciaEnDias == '' || $vigenciaEnDias == '0')
						{
							$parametroFecha = '';
						}
						else
						{
							$ExpirationDate = date('Y-m-d', strtotime(' + '.$vigenciaEnDias.' days'));
							$parametroFecha = '"ExpirationDate": "'.strval($ExpirationDate).'",';
						}
						
						$parametroPais = '';
						
						if($testmode != '1')
						{
							$parametroPais = '"country" : "'.$country.'",';
						}
						
						$ch = curl_init();
						
						curl_setopt_array($ch, array(
						  CURLOPT_URL => $urlObtenerReferencia,
						  CURLOPT_RETURNTRANSFER => true,
						  CURLOPT_ENCODING => '',
						  CURLOPT_MAXREDIRS => 10,
						  CURLOPT_TIMEOUT => 0,
						  CURLOPT_FOLLOWLOCATION => true,
						  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
						  CURLOPT_CUSTOMREQUEST => 'POST',
						  CURLOPT_POSTFIELDS =>'{'.$parametroPais.'
								"Amount": "'.strval($totalOrden).'",'.
								$parametroFecha.'
								"Value": "'.strval($ordenID).'",
								"Type": "true"
							}',
						  CURLOPT_HTTPHEADER => array(
							'authorization: '.$token,
							'Content-Type: application/json'
						  ),
						));
						
						$data = curl_exec($ch);
						curl_close($ch);
						
						$body = json_decode($data);
						
						if($body->ErrorCode != 0)
						{
							$this->smarty->assign(['success' => false, 'error' => $body->ErrorMessage]);
						}
						else
						{
							$Reference = $body->Reference;
							$barcode = $Reference;
							
							if(!$this->insertarInformacionOrden($ordenID, $ordenReference, $Reference, $barcode))
							{
								if(class_exists('Logger'))
									Logger::addLog('No se pudo guardar la referencia '.$Reference.' de la orden '.$ordenID.' en la base de datos.', 1, null, null, null, true);
							}
							
							$barcode = $this->barcode(_PS_MODULE_DIR_.$this->name .'/temp/qr_'.$ordenReference.'.png', $Reference, '90');
							
							Mail::Send(
								(int)(Configuration::get('PS_LANG_DEFAULT')),
								'paycash_en_espera_de_pago',
								Mail::l('En espera de tu pago'),
								array
								(
									'{firstname}' => Tools::nl2br($customerFirstName),
									'{lastname}' => Tools::nl2br($customerLastName),
									'{shop_name}' => Tools::nl2br(Configuration::get('PS_SHOP_NAME')),
									'{order_name}' => Tools::nl2br($ordenReference),
									'{date}' => Tools::nl2br($ordenDateAdd),
									'{total}' => Tools::nl2br(number_format($totalOrden, 2, '.', ',')),
									'{paycash_instrucciones}' => Tools::nl2br($instrucciones),
									'{paycash_referencia}' => Tools::nl2br($Reference),
									'{paycash_barcode}' => Tools::nl2br('<img alt="qr" src="'._PS_BASE_URL_.'/modules/'.$this->name.'/temp/qr_'.$ordenReference.'.png'.'"/>'),
									'{paycash_vigencia}' => Tools::nl2br($ExpirationDate),
									'{history_url}' => Tools::nl2br(_PS_BASE_URL_),
									'{orden}' => (int)($ordenID)
								),
								$email, //Email destinatario
								null, //Nombre destinatario
								null, //Email Remitente
								null //Nombre Remitente
							);
							
							$this->smarty->assign('success', true);
							$this->smarty->assign(
								'datos',
								array(
									'instrucciones' => $instrucciones,
									'referencia' => $Reference,
									'caducidad' => $ExpirationDate
								)
							);
						}
					}
				}
			}
			else
			{
				$this->smarty->assign(['success' => false, 'error' => 'No se puede proceder a conectar con el servicio de PayCash porque no fue posible leer la información del pedido creado.']);
			}
		}
		else
		{
			$this->smarty->assign(['success' => false, 'error' => 'No se puede proceder a conectar con el servicio de PayCash porque no ha completado la configuración de conexión.']);
		}
		
        return $this->fetch('module:PayCashPrestashop/views/templates/hook/payment_return.tpl');
    }

	public function hookUpdateOrderStatus($parametros)
	{
		$test_urlObtenerToken = 'https://api-pcdocs-emisor.paycashglobal.com/v1/authre';
		$produccion_urlObtenerToken = 'https://api-global-emisor.paycashglobal.com/v1/authre?country=';
		$test_urlCancelarReferencia = 'https://api-pcdocs-emisor.paycashglobal.com/v1/cancel?Reference=';
		$produccion_urlCancelarReferencia = 'https://api-global-emisor.paycashglobal.com/v1/cancel';
		
		$idEstadoOrdenCancelado = 6;
		$idEstadoOrdenReembolsado = 7;
		$idNuevoEstadoOrden = $parametros['newOrderStatus']->id;
		
		if($idNuevoEstadoOrden == $idEstadoOrdenCancelado || $idNuevoEstadoOrden == $idEstadoOrdenReembolsado)
		{
			$state = new OrderState($idNuevoEstadoOrden);
			$enviarEmailAutomatico = $state->send_email;
			
			$ordenID = (int)$parametros['id_order'];
			$informacionOrden = $this->obtenerInformacionOrden($ordenID);
			
			if(empty($informacionOrden) == false)
			{
				$orden = new Order($ordenID);
				$idCustomer = $orden->id_customer;
				$customer = new Customer($idCustomer);
				$email = $customer->email;
				$customerFirstName = $customer->firstname;
				$customerLastName = $customer->lastname;
				$ordenReference = $orden->reference;
				
				$referencia = $informacionOrden['referencia'];
				
				if(isset($referencia))
				{
					$testmode = Configuration::get('paycashps_testmode');
					$country = Configuration::get('paycashps_country');
					$apiKey = ($testmode) ? Configuration::get('paycashps_test_key') : Configuration::get('paycashps_production_key');
					
					$urlObtenerToken = $test_urlObtenerToken;
					$urlCancelarReferencia = $test_urlCancelarReferencia.$referencia;
					
					if($testmode != '1')
					{
						$urlObtenerToken = $produccion_urlObtenerToken.$country;
						$urlCancelarReferencia = $produccion_urlCancelarReferencia;
					}
					
					if($apiKey != '')
					{
						$headers = array
						(
							'Content-Type : application/json',
							'key : '.$apiKey
						);
						
						if (!function_exists('curl_version'))
						{
							if(class_exists('Logger'))
								Logger::addLog("Error al cancelar referencia PayCash ".$referencia." de la orden con ID ".$ordenID.". No se puede proceder a conectar con el servicio de PayCash porque no ha habilitado CURL para PHP.", 1, null, null, null, true);
						}
						else
						{
							$ch = curl_init();
							curl_setopt($ch, CURLOPT_URL, $urlObtenerToken);
							curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
							curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
							curl_setopt($ch, CURLOPT_HEADER, 0); 
							$data = curl_exec($ch); 
							curl_close($ch);
							
							$body = json_decode($data);
					
							if($body->ErrorCode != 0)
							{
								if(class_exists('Logger'))
									Logger::addLog("Error al cancelar referencia PayCash ".$referencia." de la orden con ID ".$ordenID.". ".$body->ErrorMessage, 1, null, null, null, true);
							}
							else
							{
								$token = $body->Authorization;
								
								$parametroPais = '';
						
								if($testmode != '1')
								{
									$parametroPais = '"country" : "'.$country.'", "Reference" : "'.$referencia.'"';
								}
								
								$ch = curl_init();
								
								curl_setopt_array($ch, array(
								  CURLOPT_URL => $urlCancelarReferencia,
								  CURLOPT_RETURNTRANSFER => true,
								  CURLOPT_ENCODING => '',
								  CURLOPT_MAXREDIRS => 10,
								  CURLOPT_TIMEOUT => 0,
								  CURLOPT_FOLLOWLOCATION => true,
								  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
								  CURLOPT_CUSTOMREQUEST => 'POST',
								  CURLOPT_POSTFIELDS =>'{'.$parametroPais.'}',
								  CURLOPT_HTTPHEADER => array(
									'authorization: '.$token,
									'Content-Type: application/json'
								  ),
								));
								
								$data = curl_exec($ch);
								curl_close($ch);
								
								$body = json_decode($data);
								
								if(!array_key_exists('ErrorCode', $body))
								{
									//ENVIO EMAIL
									if(class_exists('Logger'))
										Logger::addLog("Envio de notificacion de error en cancelacion de referencia PayCash ".$referencia." de la orden con ID ".$ordenID, 1, null, null, null, true);
									
									if(!$enviarEmailAutomatico)
									{
										Mail::Send(
											(int)(Configuration::get('PS_LANG_DEFAULT')),
											'paycash_cancelacion',
											Mail::l("Cancelación del pedido ".$ordenReference),
											array
											(
												'{firstname}' => Tools::nl2br($customerFirstName),
												'{lastname}' => Tools::nl2br($customerLastName),
												'{pedido}' => Tools::nl2br($ordenReference),
												'{titulo}' => Tools::nl2br('Su pedido con número de referencia <span class="label" style="font-weight: 700;">'.$ordenReference.'</span> de <span class="label" style="font-weight: 700;">'.Configuration::get('PS_SHOP_NAME').'</span> ha sido cancelado por el comerciante.<br/><br/>Diríjase a su cuenta de cliente para obtener más información al respecto.')
											),
											$email, //Email destinatario
											null, //Nombre destinatario
											null, //Email Remitente
											null //Nombre Remitente
										);
									}
									
									Mail::Send(
										(int)(Configuration::get('PS_LANG_DEFAULT')),
										'paycash_cancelacion',
										Mail::l('Error al cancelar referencia PayCash del pedido '.$ordenReference),
										array
										(
											'{firstname}' => Tools::nl2br(''),
											'{lastname}' => Tools::nl2br(''),
											'{pedido}' => Tools::nl2br($ordenReference),
											'{titulo}' => Tools::nl2br('Error al cancelar la referencia <span class="label" style="font-weight: 700;">'.$Reference.'</span> en PayCash del pedido '.$ordenReference.'<br/><br/><span class="label" style="font-weight: 700;">Detalle del error: </span>No se pudo obtener/leer la respuesta del servicio de cancelación de referencia de PayCash.')
										),
										Configuration::get('PS_SHOP_EMAIL'), //Email destinatario
										null, //Nombre destinatario
										null, //Email Remitente
										null //Nombre Remitente
									);
								}
								else
								{
									if($body->ErrorCode != 0)
									{
										//ENVIO EMAIL
										if(class_exists('Logger'))
											Logger::addLog("Envio de notificacion de error en cancelacion de referencia PayCash ".$referencia." de la orden con ID ".$ordenID.". ".$body->ErrorMessage, 1, null, null, null, true);
									
										if(!$enviarEmailAutomatico)
										{
											Mail::Send(
												(int)(Configuration::get('PS_LANG_DEFAULT')),
												'paycash_cancelacion',
												Mail::l("Cancelación del pedido ".$ordenReference),
												array
												(
													'{firstname}' => Tools::nl2br($customerFirstName),
													'{lastname}' => Tools::nl2br($customerLastName),
													'{pedido}' => Tools::nl2br($ordenReference),
													'{titulo}' => Tools::nl2br('Su pedido con número de referencia <span class="label" style="font-weight: 700;">'.$ordenReference.'</span> de <span class="label" style="font-weight: 700;">'.Configuration::get('PS_SHOP_NAME').'</span> ha sido cancelado por el comerciante.<br/><br/>Diríjase a su cuenta de cliente para obtener más información al respecto.')
												),
												$email, //Email destinatario
												null, //Nombre destinatario
												null, //Email Remitente
												null //Nombre Remitente
											);
										}
										
										Mail::Send(
											(int)(Configuration::get('PS_LANG_DEFAULT')),
											'paycash_cancelacion',
											Mail::l('Error al cancelar referencia PayCash del pedido '.$ordenReference),
											array
											(
												'{firstname}' => Tools::nl2br(''),
												'{lastname}' => Tools::nl2br(''),	
												'{pedido}' => Tools::nl2br($ordenReference),
												'{titulo}' => Tools::nl2br('Error al cancelar la referencia <span class="label" style="font-weight: 700;">'.$Reference.'</span> en PayCash del pedido '.$ordenReference.'<br/><br/><span class="label" style="font-weight: 700;">Código del error: </span>'.$body->ErrorCode.'<br/><br/><span class="label" style="font-weight: 700;">Detalle del error: </span>'.$body->ErrorMessage)
											),
											Configuration::get('PS_SHOP_EMAIL'), //Email destinatario
											null, //Nombre destinatario
											null, //Email Remitente
											null //Nombre Remitente
										);
									}
									else
									{
										$Reference = $body->Reference;
										$StatusName = $body->StatusName;
										
										//ENVIO EMAIL
										if(class_exists('Logger'))
											Logger::addLog("Envio de notificacion de cancelacion de referencia PayCash ".$referencia." de la orden con ID ".$ordenID.". ".$body->ErrorMessage, 1, null, null, null, true);			
									
										if(!$enviarEmailAutomatico)
										{
											Mail::Send(
												(int)(Configuration::get('PS_LANG_DEFAULT')),
												'paycash_cancelacion',
												Mail::l("Cancelación del pedido ".$ordenReference),
												array
												(
													'{firstname}' => Tools::nl2br($customerFirstName),
													'{lastname}' => Tools::nl2br($customerLastName),
													'{pedido}' => Tools::nl2br($ordenReference),
													'{titulo}' => Tools::nl2br('Su pedido con número de referencia <span class="label" style="font-weight: 700;">'.$ordenReference.'</span> de <span class="label" style="font-weight: 700;">'.Configuration::get('PS_SHOP_NAME').'</span> ha sido cancelado por el comerciante.<br/><br/>Diríjase a su cuenta de cliente para obtener más información al respecto.')
												),
												$email, //Email destinatario
												null, //Nombre destinatario
												null, //Email Remitente
												null //Nombre Remitente
											);
										}
										
										Mail::Send(
											(int)(Configuration::get('PS_LANG_DEFAULT')),
											'paycash_cancelacion',
											Mail::l("Cancelación del pedido ".$ordenReference),
											array
											(
												'{firstname}' => Tools::nl2br(''),
												'{lastname}' => Tools::nl2br(''),	
												'{pedido}' => Tools::nl2br($ordenReference),
												'{titulo}' => Tools::nl2br('La referencia <span class="label" style="font-weight: 700;">'.$Reference.'</span> del pedido <span class="label" style="font-weight: 700;">'.$ordenReference.'</span> en PayCash ha cambiado su estatus a <span class="label" style="font-weight: 700;">'.$StatusName.'</span>.')
											),
											Configuration::get('PS_SHOP_EMAIL'), //Email destinatario
											null, //Nombre destinatario
											null, //Email Remitente
											null //Nombre Remitente
										);
									}
								}
							}
						}
					}
					else
					{
						if(class_exists('Logger'))
							Logger::addLog("Error al cancelar referencia PayCash ".$referencia." de la orden con ID ".$ordenID.". No se puede proceder a conectar con el servicio de PayCash porque no ha completado la configuracion de conexion.", 1, null, null, null, true);
					}
				}
			}
		}
	}
	
	public function crearTablaOrdenReferencia()
    {
        return (Db::getInstance()->Execute
		(
            'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'paycash_orden_informacion` (
				`id_orden_paycash` INT NOT NULL AUTO_INCREMENT,
				`id_orden` INT NOT NULL,
				`referencia_orden` VARCHAR(200) NOT NULL,
				`referencia` VARCHAR(200) NOT NULL,
				`barcode` VARCHAR(200) NOT NULL,
				`paid_at` VARCHAR(200) NOT NULL,
				`charge` VARCHAR(200) NOT NULL,
				`payment_method` VARCHAR(200) NOT NULL,
				PRIMARY KEY (`id_orden_paycash`)
			)
			ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;'
        ));
    }
	
	public function obtenerInformacionOrden($id)
    {
        return Db::getInstance()->getRow
		(
            'SELECT * FROM '._DB_PREFIX_.'paycash_orden_informacion WHERE id_orden = '.pSQL((int)$id).';'
        );
    }
	
	public function insertarInformacionOrden($idOrden, $referenciaOrden, $referencia, $barcode)
    {
        return Db::getInstance()->Execute
		(
            'INSERT INTO '._DB_PREFIX_.'paycash_orden_informacion (id_orden, referencia_orden, referencia, barcode, paid_at, charge, payment_method) VALUES ('.pSQL((int)$idOrden).', "'.pSQL($referenciaOrden).'", "'.pSQL($referencia).'", "'.pSQL($barcode).'", "", "", "");'
        );
    }
	
	function barcode( $filepath="", $text="0", $size="20", $orientation="horizontal", $code_type="code128", $print=false, $SizeFactor=1 ) {
		$code_string = "";
		// Translate the $text into barcode the correct $code_type
		if ( in_array(strtolower($code_type), array("code128", "code128b")) ) {
			$chksum = 104;
			// Must not change order of array elements as the checksum depends on the array's key to validate final code
			$code_array = array(" "=>"212222","!"=>"222122","\""=>"222221","#"=>"121223","$"=>"121322","%"=>"131222","&"=>"122213","'"=>"122312","("=>"132212",")"=>"221213","*"=>"221312","+"=>"231212",","=>"112232","-"=>"122132","."=>"122231","/"=>"113222","0"=>"123122","1"=>"123221","2"=>"223211","3"=>"221132","4"=>"221231","5"=>"213212","6"=>"223112","7"=>"312131","8"=>"311222","9"=>"321122",":"=>"321221",";"=>"312212","<"=>"322112","="=>"322211",">"=>"212123","?"=>"212321","@"=>"232121","A"=>"111323","B"=>"131123","C"=>"131321","D"=>"112313","E"=>"132113","F"=>"132311","G"=>"211313","H"=>"231113","I"=>"231311","J"=>"112133","K"=>"112331","L"=>"132131","M"=>"113123","N"=>"113321","O"=>"133121","P"=>"313121","Q"=>"211331","R"=>"231131","S"=>"213113","T"=>"213311","U"=>"213131","V"=>"311123","W"=>"311321","X"=>"331121","Y"=>"312113","Z"=>"312311","["=>"332111","\\"=>"314111","]"=>"221411","^"=>"431111","_"=>"111224","\`"=>"111422","a"=>"121124","b"=>"121421","c"=>"141122","d"=>"141221","e"=>"112214","f"=>"112412","g"=>"122114","h"=>"122411","i"=>"142112","j"=>"142211","k"=>"241211","l"=>"221114","m"=>"413111","n"=>"241112","o"=>"134111","p"=>"111242","q"=>"121142","r"=>"121241","s"=>"114212","t"=>"124112","u"=>"124211","v"=>"411212","w"=>"421112","x"=>"421211","y"=>"212141","z"=>"214121","{"=>"412121","|"=>"111143","}"=>"111341","~"=>"131141","DEL"=>"114113","FNC 3"=>"114311","FNC 2"=>"411113","SHIFT"=>"411311","CODE C"=>"113141","FNC 4"=>"114131","CODE A"=>"311141","FNC 1"=>"411131","Start A"=>"211412","Start B"=>"211214","Start C"=>"211232","Stop"=>"2331112");
			$code_keys = array_keys($code_array);
			$code_values = array_flip($code_keys);
			for ( $X = 1; $X <= strlen($text); $X++ ) {
				$activeKey = substr( $text, ($X-1), 1);
				$code_string .= $code_array[$activeKey];
				$chksum=($chksum + ($code_values[$activeKey] * $X));
			}
			$code_string .= $code_array[$code_keys[($chksum - (intval($chksum / 103) * 103))]];

			$code_string = "211214" . $code_string . "2331112";
		} elseif ( strtolower($code_type) == "code128a" ) {
			$chksum = 103;
			$text = strtoupper($text); // Code 128A doesn't support lower case
			// Must not change order of array elements as the checksum depends on the array's key to validate final code
			$code_array = array(" "=>"212222","!"=>"222122","\""=>"222221","#"=>"121223","$"=>"121322","%"=>"131222","&"=>"122213","'"=>"122312","("=>"132212",")"=>"221213","*"=>"221312","+"=>"231212",","=>"112232","-"=>"122132","."=>"122231","/"=>"113222","0"=>"123122","1"=>"123221","2"=>"223211","3"=>"221132","4"=>"221231","5"=>"213212","6"=>"223112","7"=>"312131","8"=>"311222","9"=>"321122",":"=>"321221",";"=>"312212","<"=>"322112","="=>"322211",">"=>"212123","?"=>"212321","@"=>"232121","A"=>"111323","B"=>"131123","C"=>"131321","D"=>"112313","E"=>"132113","F"=>"132311","G"=>"211313","H"=>"231113","I"=>"231311","J"=>"112133","K"=>"112331","L"=>"132131","M"=>"113123","N"=>"113321","O"=>"133121","P"=>"313121","Q"=>"211331","R"=>"231131","S"=>"213113","T"=>"213311","U"=>"213131","V"=>"311123","W"=>"311321","X"=>"331121","Y"=>"312113","Z"=>"312311","["=>"332111","\\"=>"314111","]"=>"221411","^"=>"431111","_"=>"111224","NUL"=>"111422","SOH"=>"121124","STX"=>"121421","ETX"=>"141122","EOT"=>"141221","ENQ"=>"112214","ACK"=>"112412","BEL"=>"122114","BS"=>"122411","HT"=>"142112","LF"=>"142211","VT"=>"241211","FF"=>"221114","CR"=>"413111","SO"=>"241112","SI"=>"134111","DLE"=>"111242","DC1"=>"121142","DC2"=>"121241","DC3"=>"114212","DC4"=>"124112","NAK"=>"124211","SYN"=>"411212","ETB"=>"421112","CAN"=>"421211","EM"=>"212141","SUB"=>"214121","ESC"=>"412121","FS"=>"111143","GS"=>"111341","RS"=>"131141","US"=>"114113","FNC 3"=>"114311","FNC 2"=>"411113","SHIFT"=>"411311","CODE C"=>"113141","CODE B"=>"114131","FNC 4"=>"311141","FNC 1"=>"411131","Start A"=>"211412","Start B"=>"211214","Start C"=>"211232","Stop"=>"2331112");
			$code_keys = array_keys($code_array);
			$code_values = array_flip($code_keys);
			for ( $X = 1; $X <= strlen($text); $X++ ) {
				$activeKey = substr( $text, ($X-1), 1);
				$code_string .= $code_array[$activeKey];
				$chksum=($chksum + ($code_values[$activeKey] * $X));
			}
			$code_string .= $code_array[$code_keys[($chksum - (intval($chksum / 103) * 103))]];

			$code_string = "211412" . $code_string . "2331112";
		} elseif ( strtolower($code_type) == "code39" ) {
			$code_array = array("0"=>"111221211","1"=>"211211112","2"=>"112211112","3"=>"212211111","4"=>"111221112","5"=>"211221111","6"=>"112221111","7"=>"111211212","8"=>"211211211","9"=>"112211211","A"=>"211112112","B"=>"112112112","C"=>"212112111","D"=>"111122112","E"=>"211122111","F"=>"112122111","G"=>"111112212","H"=>"211112211","I"=>"112112211","J"=>"111122211","K"=>"211111122","L"=>"112111122","M"=>"212111121","N"=>"111121122","O"=>"211121121","P"=>"112121121","Q"=>"111111222","R"=>"211111221","S"=>"112111221","T"=>"111121221","U"=>"221111112","V"=>"122111112","W"=>"222111111","X"=>"121121112","Y"=>"221121111","Z"=>"122121111","-"=>"121111212","."=>"221111211"," "=>"122111211","$"=>"121212111","/"=>"121211121","+"=>"121112121","%"=>"111212121","*"=>"121121211");

			// Convert to uppercase
			$upper_text = strtoupper($text);

			for ( $X = 1; $X<=strlen($upper_text); $X++ ) {
				$code_string .= $code_array[substr( $upper_text, ($X-1), 1)] . "1";
			}

			$code_string = "1211212111" . $code_string . "121121211";
		} elseif ( strtolower($code_type) == "code25" ) {
			$code_array1 = array("1","2","3","4","5","6","7","8","9","0");
			$code_array2 = array("3-1-1-1-3","1-3-1-1-3","3-3-1-1-1","1-1-3-1-3","3-1-3-1-1","1-3-3-1-1","1-1-1-3-3","3-1-1-3-1","1-3-1-3-1","1-1-3-3-1");

			for ( $X = 1; $X <= strlen($text); $X++ ) {
				for ( $Y = 0; $Y < count($code_array1); $Y++ ) {
					if ( substr($text, ($X-1), 1) == $code_array1[$Y] )
						$temp[$X] = $code_array2[$Y];
				}
			}

			for ( $X=1; $X<=strlen($text); $X+=2 ) {
				if ( isset($temp[$X]) && isset($temp[($X + 1)]) ) {
					$temp1 = explode( "-", $temp[$X] );
					$temp2 = explode( "-", $temp[($X + 1)] );
					for ( $Y = 0; $Y < count($temp1); $Y++ )
						$code_string .= $temp1[$Y] . $temp2[$Y];
				}
			}

			$code_string = "1111" . $code_string . "311";
		} elseif ( strtolower($code_type) == "codabar" ) {
			$code_array1 = array("1","2","3","4","5","6","7","8","9","0","-","$",":","/",".","+","A","B","C","D");
			$code_array2 = array("1111221","1112112","2211111","1121121","2111121","1211112","1211211","1221111","2112111","1111122","1112211","1122111","2111212","2121112","2121211","1121212","1122121","1212112","1112122","1112221");

			// Convert to uppercase
			$upper_text = strtoupper($text);

			for ( $X = 1; $X<=strlen($upper_text); $X++ ) {
				for ( $Y = 0; $Y<count($code_array1); $Y++ ) {
					if ( substr($upper_text, ($X-1), 1) == $code_array1[$Y] )
						$code_string .= $code_array2[$Y] . "1";
				}
			}
			$code_string = "11221211" . $code_string . "1122121";
		}

		// Pad the edges of the barcode
		$code_length = 20;
		if ($print) {
			$text_height = 30;
		} else {
			$text_height = 0;
		}
		
		for ( $i=1; $i <= strlen($code_string); $i++ ){
			$code_length = $code_length + (integer)(substr($code_string,($i-1),1));
			}

		if ( strtolower($orientation) == "horizontal" ) {
			$img_width = $code_length*$SizeFactor;
			$img_height = $size;
		} else {
			$img_width = $size;
			$img_height = $code_length*$SizeFactor;
		}

		$image = imagecreate($img_width, $img_height + $text_height);
		$black = imagecolorallocate ($image, 0, 0, 0);
		$white = imagecolorallocate ($image, 255, 255, 255);

		imagefill( $image, 0, 0, $white );
		if ( $print ) {
			imagestring($image, 5, 31, $img_height, $text, $black );
		}

		$location = 10;
		for ( $position = 1 ; $position <= strlen($code_string); $position++ ) {
			$cur_size = $location + ( substr($code_string, ($position-1), 1) );
			if ( strtolower($orientation) == "horizontal" )
				imagefilledrectangle( $image, $location*$SizeFactor, 0, $cur_size*$SizeFactor, $img_height, ($position % 2 == 0 ? $white : $black) );
			else
				imagefilledrectangle( $image, 0, $location*$SizeFactor, $img_width, $cur_size*$SizeFactor, ($position % 2 == 0 ? $white : $black) );
			$location = $cur_size;
		}
		
		// Draw barcode to the screen or save in a file
		if ( $filepath=="" ) {
			header ('Content-type: image/png');
			imagepng($image);
			imagedestroy($image);
		} else {
			imagepng($image,$filepath);
			imagedestroy($image);		
		}
	}
}